function [ val ] = cheb_poly_reco( freq, z, M, fct_handle )
%CHEB_POLY_RECO Fast reconstruction of Chebyshev coefficients of multivariate
%algebraic polynomial in Chebyshev basis with frequencies supported on
%index set freq from samples taken at nodes x_j of reconstructing rank-1
%Chebyshev lattice.
%
%   This means that the sampling nodes are then given by
%   x_j := (cos(pi*j*z(1)/M),...,cos(pi*j*z(d))/M),
%   j=0,...,M.
%
%   Input parameters:
%      freq - frequency index set as matrix of size [num_freq,d]
%      z - generating vector of rank-1 Chebyshev lattice as a row vector of
%          length d
%      M - size parameter of rank-1 Chebyshev lattice
%      fct_handle - function handle (block box) which accepts sampling
%                   nodes from [-1,1]^d as an array of size [num_nodes,d]
%
%    Ouput parameters:
%       val - reconstructed Chebyshev coefficients
%
% Copyright (c) 2015 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

%if ~dct_r1l_check(freq,z,M); error('not reco'); end
d = length(z);
if isa(fct_handle,'function_handle')
  nodes = cos((0:M)'*z*pi/M);
  f_samples = fct_handle(nodes);
  clear nodes;
else
  f_samples = fct_handle;
end

f_hat = fct1(f_samples) * 2/M;
if M >= 1
  f_hat(1) = f_hat(1)/2;
  f_hat(end) = f_hat(end)/2;
end

s=1;
m_s = [1;-1];
for t=2:d
  m_s = [reshape(repmat([1,-1], size(m_s,1), 1), 2*size(m_s,1), 1) repmat(m_s, 2, 1)];
end
m_s(m_s(:,s) < 1,:) = [];

%factor = repmat(2^(d-1),size(freq,1),1);
factor = ones(size(freq,1),1);
for ik=1:size(freq,1)
  k = freq(ik,:);
  k_m = repmat(k,size(m_s,1),1) .* m_s;
%  k_m_z_emodM = emod(k_m*z',M);
%  factor(ik) = factor(ik) / 2^(length(find(k_m_z_emodM==emod(k*z',M)))-1);
  factor(ik) = 2^(d-1) / length(find(emod(k_m*z',M)==emod(k*z',M)));
end

val = factor .* f_hat(emod(freq*z',M)+1);

end

