function [ z ] = dct_cbc_lattice_search( freq, M, index_s, opts, z_in )
%DCT_CBC_LATTICE_SEARCH Searches for generating vector z of reconstructing rank-1 Chebychev lattice.
%
%   Input parameters:
%      freq - frequency index set as a matrix of size [num_freq,d]
%      M - size parameter
%      index_s - index of dimension where index set is not mirrored
%      opts - struct of optional parameters
%             opts.Display:   
%                'off': no information is shown
%                'cbc': displays the components when being found
%             opts.Inc:
%                0: search for each component starts from 1
%                1: search for each component z_s starts from z_{s-1}+1,
%                   result is an ordered vector [usefull for equalweighted
%                   hyperbolic crosses or l1balls]
%             opts.Bisection_Successors:
%                number of successors to test (default 1000)
%      z_in: (optional) minimal values for z
%
%   Output parameters:
%      z - generating vector as a row vector of length d of reconstructing rank-1 Chebyshev lattice,
%          note that length is less than d if none was found
%
% Copyright (c) 2014 Lutz Kaemmerer, 2015 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

if nargin < 3
  index_s = 1;
end

if nargin < 4
    opts.Display='off';
    opts.Inc=0;
else
    if(~(isfield(opts,'Inc') && opts.Inc==1))
        opts.Inc=0;
    end
    if(~(isfield(opts,'Display') && strmatch(opts.Display, char('off', 'cbc', 'final'),'exact')>0))
        opts.Display='off';
    end
end

d = size(freq,2);

if nargin < 5
    z_in = [];
end

if length(z_in) < d
    z_in = [z_in, zeros(1, d-length(z_in))];
end

ticsearch=tic;
z = [];
for s=1:size(freq,2)
  cur_index_s = index_s;
  if cur_index_s > s
    cur_index_s = s;
  end
  freq_s = unique(freq(:,1:s),'rows');

  tmpstart=0;
  switch(opts.Inc)
    case 1
      tmpstart = z(s-1)+1;
  end
  tmpstart = max(tmpstart, z_in(s));
  for z_s=tmpstart:M
    z_test = [z,z_s];
    if dct_r1l_check(freq_s,z_test,M,cur_index_s)
      z = [z,z_s];
      break;
    end
  end
  if(strcmp(opts.Display,'cbc')==1)
      fprintf('dimension %d, z_%d=%d\n', [s,s,z(s)])
  end
end
t1=toc(ticsearch);

if (strcmp(opts.Display,'final')==1) || (strcmp(opts.Display,'cbc')==1)
  strtoshow=sprintf('generating vector z=[%d',z(1));
  for s=2:length(z)
    strtoshow=[strtoshow,',%d'];
  end
  strtoshow=sprintf([strtoshow,']\n'],z(2:length(z)));
  strtoshow=[strtoshow,sprintf('cbc search time: %d',t1)];
  disp(strtoshow);
end


end

