function [ M_out ] = dct_lattice_size_reduce( freq, M, z, index_s, opts )
%DCT_LATTICE_SIZE_REDUCE Searches for reconstructing rank-1 Chebyshev lattice of smaller size.
%   The reconstructing property is checked by calling the dct_r1l_check function.
%   Input parameters:
%      freq - frequency index set as a matrix of size [num_freq,d]
%      M - best-known size parameter of reconstructing rank-1 Chebyshev lattice for index set freq
%      z - generating vector of reconstructing rank-1 Chebyshev lattice for index set freq
%      index_s - index of dimension where index set is not mirrored
%      opts - struct of optional parameters
%             opts.Display:   
%                'off': no display
%                'iter': display current M
%             opts.Prime:
%                1: only primes
%                0: all size parameters < input M
%
%   Output parameters:
%      M_out - new size parameter of reconstructing rank-1 Chebyshev lattice
%
% Copyright (c) 2014 Lutz Kaemmerer, 2015 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
%   Detailed explanation goes here

if nargin < 4
  index_s = 1;
end

if nargin < 5
    opts.Display='off';
    opts.Prime=0;
    opts.Mmin = size(freq,1)-1;
else
    if(~(isfield(opts,'Prime') && opts.Prime==1))
        opts.Prime=0;
    end
    if ~(isfield(opts,'Display') && (strcmp(opts.Display, 'off') || strcmp(opts.Display, 'iter')))
        opts.Display='off';
    end
    if ~isfield(opts,'Mmin')
        opts.Mmin = size(freq,1)-1;
    end
end    


freq = unique(freq,'rows');

if opts.Prime == 1
  M_out = simplenextprime(opts.Mmin-1);
else
  M_out = opts.Mmin;
end

tic
while M_out < M
  if dct_r1l_check(freq,z,M_out,index_s)
    break;
  end

  if toc > 10
    if(strcmp(opts.Display,'iter')==1)
      fprintf('curr candidate: %d, max M: %d\n', M_out, M);
    end
    tic
  end

  if opts.Prime == 1
    M_out = simplenextprime(M_out);
  else
    M_out = M_out + 1;
  end
end

toc;

if (~dct_r1l_check(freq,z,M_out,index_s)) && (M_out == M)
    error('no r1l found');
end

if M_out > M
  M_out = M;
end

if ~dct_r1l_check(freq,z,M_out,index_s)
  error('no r1l found');
end

end

function [y] = simplenextprime(n)
% computes the next prime larger n
% uses simple isprime function of MATLAB
% Author: Lutz KÃ¤mmerer

y=n+1;
if mod(y,2)==0,
   y=y+1;
end;

flag=1;
while flag,
   if(isprime(y)),
      return;
   end;
   y=y+2;
end;
end
