function [M]=dct_lattice_size_reduce_bisect_only(h,Mmax,z,opts)
%DCT_LATTICE_SIZE_REDUCE_BISECT_ONLY Searches for reconstructing rank-1 Chebyshev lattice of smaller size.
%   This function uses only a bisection method. The reconstructing property
%   is checked by calling the dct_r1l_check function.
%   Input parameters:
%      h - frequency index set as a matrix of size [num_freq,d]
%      Mmax - best-known size of reconstructing rank-1 lattice for index set h
%      z - generating vector of reconstructing rank-1 lattice for index set h
%      opts - struct of optional parameters
%             opts.Display:   
%                'off': no display
%                'iter': display actual M
%             opts.Prime:
%                1: only primes
%                0: all M<Mmax
%             opts.Bisection_Successors:
%                number of successors to test (default 1000)
%
%   Output parameters:
%      M - new size of reconstructing rank-1 lattice <= Mmax
%
% Copyright (c) 2014 Lutz Kaemmerer, 2015 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

h = double(h);
hz=h*z';

Mmax=min(Mmax,max(hz)-min(hz)+1);

if max(abs(h*z')) >= 2^53
  error('values higher than double precision');
end

if Mmax >= 2^53
  error('values higher than double precision');
end

if(nargin==3)
    opts.Display='off';
    opts.Prime=0;
    opts.Bisection_Successors=1000;
else
    if(~(isfield(opts,'Prime') && opts.Prime==1))
        opts.Prime=0;
    end
    if ~(isfield(opts,'Display') && (strcmp(opts.Display, 'off') || strcmp(opts.Display, 'iter')))
        opts.Display='off';
    end
    if(~isfield(opts,'Bisection_Successors'))
        opts.Bisection_Successors=1000;
    end
end    

switch(opts.Prime)
    case 0
        j_u = Mmax+1;
        j_o = length(hz);
    case 1
        p = primes(simplenextprime(Mmax));
        candidates = p(p >= size(h,1));
        j_u = length(candidates);
        j_o = 0;
end

j = floor((j_u+j_o)/2);
lastSuccess = -1;
first_loop = 1;
while (first_loop || (j > j_o && j < j_u))
      found = 0;

      first_loop = 0;

      if j_u - j_o < opts.Bisection_Successors
        j = j_o;
      end

      r_max = min(j_u-1, j+opts.Bisection_Successors);

      for r = j:r_max
          if (opts.Prime==1 && r >= length(candidates))
              break
          end


          if opts.Prime == 1
              curM = candidates(r+1);
          else
              curM = r;
          end
          
          if curM > Mmax
            break;
          end
          
          if dct_r1l_check(h,z,curM)
              lastSuccess = r;
	          found = 1;
              break;
          end
      end

      if found
        j_u = j;
      end

      if j_u ~= j
        j_o = j + opts.Bisection_Successors;
      end
      j = floor((j_u+j_o)/2);
end

if (opts.Prime==1 && (j_o >= length(candidates)))
%     error('no better prime rank-1 lattice size M found');
    M = Mmax;
    return;
end

if (opts.Prime == 1 && lastSuccess >= 0)
    lastSuccess = candidates(lastSuccess+1);
end

if lastSuccess > 0
    M = lastSuccess;
else
    M = Mmax;
end

end

function [y] = simplenextprime(n)
% computes the next prime larger n
% uses simple isprime function of MATLAB
% Author: Lutz Kämmerer

y=n+1;
if mod(y,2)==0,
   y=y+1;
end;

flag=1;
while flag,
   if(isprime(y)),
      return;
   end;
   y=y+2;
end;
end