function [ M ] = lfft_lattice_size_reduce(h,Mmax,z,opts)
%LFFT_LATTICE_SIZE_REDUCE Searches for reconstructing rank-1 lattice of smaller size.
%   Input parameters:
%      h - frequency index set as a matrix of size [num_freq,d]
%      Mmax - best-known size of reconstructing rank-1 lattice for index set h
%      z - generating vector of reconstructing rank-1 lattice for index set h
%      opts - struct of optional parameters
%             opts.Display:   
%                'off': no display
%                'iter': display actual M
%             opts.Prime:
%                1: only primes
%                0: all M<Mmax
%             opts.Bisection_Successors:
%                number of successors to test (default 1000)
%
%   Output parameters:
%      M - new size of reconstructing rank-1 lattice <= Mmax
%
% Copyright (c) 2014 Lutz Kaemmerer, 2015 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

hz=double(h)*z';

%Mmax=min(Mmax,max(hz)-min(hz)+1);

if max(abs(h*z')) >= 2^53
  error('values higher than double precision');
end

if Mmax >= 2^53
  error('values higher than double precision');
end

if(nargin==3)
    opts.Display='off';
    opts.Prime=0;
    opts.Bisection_Successors=1000;
    opts.Mmin = size(h,1);
else
    if(~(isfield(opts,'Prime') && opts.Prime==1))
        opts.Prime=0;
    end
    if ~(isfield(opts,'Display') && (strcmp(opts.Display, 'off') || strcmp(opts.Display, 'iter')))
        opts.Display='off';
    end
    if(~isfield(opts,'Bisection_Successors'))
        opts.Bisection_Successors=1000;
    end
    if ~isfield(opts,'Mmin')
        opts.Mmin = size(h,1);
    end
end    

switch(opts.Prime)
    case 0
        j_o = opts.Mmin;
    case 1
        p = primes(simplenextprime(Mmax));
        candidates = p(p >= opts.Mmin);
        j_o = 0;
end

M = Mmax;

tDisplay = tic;
for j=1:j_o+1
    if opts.Prime==1
        if j > length(candidates)
            break;
        end
        curM = candidates(j);
    else
        curM = opts.Mmin-1+j;
    end
    if curM > M
        break;
    end
    if(length(unique(mod(hz,curM)))==size(h,1))
        M = curM;
        break;
    end
    elapsed_time = toc(tDisplay);
    if elapsed_time > 10
      if(strcmp(opts.Display,'iter')==1)
        fprintf('curr candidate: %d, max M: %d\n', curM, M);
      end
      tDisplay = tic;
    end
end
%elapsed_time = toc(tDisplay);
end

function [y] = simplenextprime(n)
% computes the next prime larger n
% uses simple isprime function of MATLAB
% Author: Lutz Kämmerer

y=n+1;
if mod(y,2)==0,
   y=y+1;
end;

flag=1;
while flag,
   if(isprime(y)),
      return;
   end;
   y=y+2;
end;
end
