function [ freq, val ] = reco_cardI_cl_basic1( fct_handle, d, N, dim, threshold )
%RECO_CARDI_CL_BASIC1 Utility function for the function sfft_cl_algorithm.
%   This function realizes step 1 and 2a of Algorithm 1/2.
%   The one-dimensionally projected Chebyshev coefficients, which are
%   projected to the component specificied by the parameter dims,
%   and the projected Chebyshev coefficients larger than the relative
%   threshold are returned with the corresponding frequencies.
%
%   Input parameters:
%      fct_handle - function handle (block box) which accepts sampling
%                   nodes as an array of size [num_nodes,d]
%      d - dimensionality
%      N - parameter for search space Gamma = [0,...,N]^d of possible
%          frequencies
%      threshold - (optional) relative threshold theta used for truncation
%      dim - the component to project to
%      threshold - relative threshold used for truncation.
%
%    Ouput parameters:
%       freq - frequencies belonging to the projected Chebyshev coefficients
%              larger than the relative threshold
%       val - projected Chebyshev coefficients larger than the relative
%             threshold
%
% Copyright (c) 2015 Toni Volkmer
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 2 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
% FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, write to the Free Software Foundation, Inc., 51
% Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

if d < 1
  error('d must be at least 1');
end

if nargin < 5
  threshold = 1e-12;
end

if length(dim) ~= 1
  error('parameter dim has to have length 1');
end

if (dim < 1) || (dim > d)
  error('parameter dim has to fulfill 1 <= dim <= d');
end

freq_cand = (0:N)';

x = repmat(2*rand(1,d)-1,N+1,1);
if N >= 1
  x(:,dim) = cos(freq_cand * pi / N);
else
  x(:,dim) = 1;
end
samples = fct_handle(x);
clear x;

val_all = fct1(samples) * 2/N;
if N >= 1
  val_all(1) = val_all(1)/2;
  val_all(end) = val_all(end)/2;
end

max_abs_val_all = max(abs(val_all));
absolute_threshold = threshold*max_abs_val_all;
ind = find(abs(val_all) >= absolute_threshold);
freq = freq_cand(ind,:);
val = val_all(ind);

end

